#ifndef 	LCD_C_
#define		LCD_C_

#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<stdarg.h>
#include 	<string.h>
#include 	<avr/pgmspace.h>
#include 	"mydefs.h"
#include 	"lcd.h"
//---------------------------------------------------------------------------------//
// Delay time little
// jnut comment 8MHZ  _delay_us (2);
// enough
void lcd_delay (void)
{
	_delay_ms (1); //Clock 16 MHZ delay about 100 us enough 
}
//---------------------------------------------------------------------------------//
// Toggle High and Low Enable Pin
void toggle_e(void)
{
    LCD_CTRL_PORT |= _BV(LCD_E);
	lcd_delay ();
    LCD_CTRL_PORT &= ~_BV(LCD_E);
	lcd_delay ();
};
//---------------------------------------------------------------------------------//
//Read Busy From LCD
#ifdef 	USE_RW_PIN	
uint8_t lcd_read (void)
{
	uint8_t dat;

		LCD_Port_DDR    = LCD_DataInput;//  | 0x0F;
		
		LCD_CTRL_PORT  &= ~_BV(LCD_RS);			// Read Status
		LCD_CTRL_PORT  |= _BV(LCD_RW);
		LCD_CTRL_PORT  |= _BV(LCD_E);				// Read...
		_delay_us (150);
		
		LCD_CTRL_PORT &= ~_BV(LCD_E);
		_delay_us (150);
		dat = LCD_Port_Read & 0xF0; 				// 1. Nibble (MSB)

	LCD_Port_DDR	= 	LCD_DataOutput;// | 0x0F;
	LCD_CTRL_PORT   &=	~_BV(LCD_RW);		// RW-Pin = Low
	return dat;
}
#endif
//---------------------------------------------------------------------------------//
//Write Command to LCD
void LCD_Write_cmd (char cmd)
{
#ifdef 	USE_RW_PIN	
		while (!(lcd_read() & 0x80));
#else
		_delay_ms (1);
#endif	
	LCD_Port_Write &= ~_BV(LCD_RS);
	LCD_Port_Write &= ~_BV(LCD_E);
	LCD_Port_Write  = ((LCD_Port_Write & LCD_BUSALL) |  (cmd & 0xF0)); 	//Write Nibbel MSB
	toggle_e ();
		
	LCD_Port_Write  = ((LCD_Port_Write & LCD_BUSALL) | (cmd & 0x0F) << 4); //Write Nibbel LSB
		
	toggle_e ();		
}
//---------------------------------------------------------------------------------//
//Write Data to LCD
void LCD_Write_Char (char dat)
{
#ifdef 	USE_RW_PIN	
		while (!(lcd_read() & 0x80));
#else
	_delay_ms (1);
#endif		
	LCD_CTRL_PORT |= 	_BV(LCD_RS);
	LCD_CTRL_PORT &=  ~_BV(LCD_E);
	
	LCD_Port_Write  = ((LCD_Port_Write & LCD_BUSALL) | (dat & 0xF0)); 		//Write Nibbel MSB
	toggle_e ();		
	
	LCD_Port_Write  = ((LCD_Port_Write & LCD_BUSALL) | (dat & 0x0F) << 4); //Write Nibbel LSB
	toggle_e ();		
}
//---------------------------------------------------------------------------------//
//Set Row and Culume 
void lcd_gotoxy (uint8_t row, uint8_t col)
{
	switch (row)
	{
		case 1 : 	LCD_Write_cmd (0x80 + col -1); 
					break;
		case 2 : 	LCD_Write_cmd (0xC0 + col -1); 
					break;
		default : 	
					break;
	}
}
//---------------------------------------------------------------------------------//
//Print 1 Data to lcd
void lcd_putc (char c)
{
	switch (c)
	{
		case '\n'  : 	lcd_gotoxy (2,1);
						break;
		case '\b'	:	LCD_Write_cmd (0x10);
						break;
		default  	:	LCD_Write_Char (c);
						break;
		
	}
}
//---------------------------------------------------------------------------------//
//Write String to lcd
void lcd_printf (uint8_t *string)
{
	uint8_t len = 0;  //16 Char on LCD
	
		while ((*string) && (len++ < MAXLENGTH))
			lcd_putc (*string++);
#ifdef 	USE_RW_PIN		
	LCD_CTRL_PORT |= _BV(LCD_RW);	
#endif	
}
//---------------------------------------------------------------------------------//
//Clear Line 
void lcd_clear (uint8_t line)
{
	if (line == LINE1)
		lcd_gotoxy (LINE1,1);
	else
		if (line == LINE2)
			lcd_gotoxy (LINE2,1);
	lcd_printf("                "); //about 16 char
	lcd_gotoxy (line,1);
}
//---------------------------------------------------------------------------------//
//function init LCD
void init_lcd (void)
{
	LCD_BLDDR	|=	_BV(LCD_BL);

	
#ifdef 	USE_RW_PIN
	LCD_Port_DDR   =  LCD_DataOutput;
	LCD_CTRL_DDR   = _BV(LCD_RS) | _BV(LCD_RW) | _BV(LCD_E);
	LCD_CTRL_PORT  &= ~_BV(LCD_RW);	//jnut test use RW to GND

	LCD_Write_cmd (0x22);				// Init 4 Bit Mode 
	LCD_Write_cmd (0x80);   			// Set DDRam Address 0;
#else
	LCD_Port_DDR   =  LCD_DataOutput;
	LCD_Port_DDR   =  _BV(LCD_RS) | _BV(LCD_RW) | _BV(LCD_E));
	LCD_CTRL_PORT  &= ~_BV(LCD_RW);	//jnut test use RW to GND
#endif	
	LCD_Write_cmd (0x33);				// Init 4 Bit Mode 
	LCD_Write_cmd (0x32);   			// Set DDRam Address 0;

	LCD_Write_cmd (LCD_FUNCTION_SET);	// Function
	LCD_Write_cmd (LCD_ON);				// 5*7 , Blink
	LCD_Write_cmd (LCD_MODE_SET);		// mode
	LCD_Write_cmd (LCD_CLEAR);			// clear 
	lcd_delay ();
	LCD_BLPORT	|=	_BV(LCD_BL);		//on blacklight
}
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
void Symbol (char c, const char *data) 
{
	uint8_t i;
	
	LCD_Write_cmd (0x40 + ((c&0x07)<<3)); //Set CG-RAM address
  
	for (i=0; i<8; i++) 
	{
		LCD_Write_Char(pgm_read_byte(&data[i]));
	}
}
#endif
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
//---------------------------------------------------------------------------------//
