#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<stdio.h>
#include 	<stdarg.h>
#include 	<string.h>
#include 	<stdlib.h>

#include 	"serial.h"
#include 	"printf.h"
#include	"lcd.h"
#include 	"mydefs.h"

const char a_[] PROGMEM = {0x1F,0x10,0x10,0x10,0x10,0x10,0x10,0x1F}; //1
const char b_[] PROGMEM = {0x1F,0x00,0x00,0x00,0x00,0x00,0x00,0x1F}; //2
const char c_[] PROGMEM = {0x1C,0x04,0x03,0x03,0x03,0x03,0x04,0x1C}; //3
const char d_[] PROGMEM = {0x1F,0x10,0x17,0x17,0x17,0x17,0x10,0x1F};
const char e_[] PROGMEM = {0x1F,0x00,0x1F,0x1F,0x1F,0x1F,0x00,0x1F};
const char f_[] PROGMEM = {0x1C,0x04,0x1B,0x1B,0x1B,0x1B,0x04,0x1C};


uint8_t batt,
		flag;
uint8_t blink,nCounter;

#define 	FLAG_SEC		0x01
#define 	FLAG_BATTERY	0x02

#define 	FLAG_PRECHARGE	0x10
#define 	FLAG_CC_CHARGE	0x20
#define 	FLAG_CV_CHARGE	0x40
#define 	FLAG_FULL		0x80

#define 	VOLT			1
#define 	CURRENT			0
#define 	TIMEOUT			1

//------------------------------------------------------------------------//
void icon (void)
{
	Symbol (0,a_);
	Symbol (1,b_);
	Symbol (2,c_);
	
	Symbol (3,d_);
	Symbol (4,e_);
	Symbol (5,f_);
	
}
//------------------------------------------------------------------------//
void display_icon (uint8_t state)
{
	if (flag & FLAG_BATTERY)
	{
		lcd_gotoxy (1,1);
	
		switch (state)
		{
			case 0:	LCD_Write_Char (0);LCD_Write_Char (1);	LCD_Write_Char (1);	LCD_Write_Char (2);
				break;
			case 2:LCD_Write_Char (3);LCD_Write_Char (1);	LCD_Write_Char (1);	LCD_Write_Char (2);
				break;
			case 3:LCD_Write_Char (3);LCD_Write_Char (4);	LCD_Write_Char (1);	LCD_Write_Char (2);
				break;
			case 4:LCD_Write_Char (3);LCD_Write_Char (4);	LCD_Write_Char (4);	LCD_Write_Char (2);
				break;
			case 5:LCD_Write_Char (3);LCD_Write_Char (4);	LCD_Write_Char (4);	LCD_Write_Char (5);
				break;
			case 6:  batt= 0;
				  break;
			default :	break;
		}
	}
}
//------------------------------------------------------------------------//
//------------------------------------------------------------------------//
//-------------------------------------------------------------------------//
ISR (TIMER0_OVF_vect)
{
	if (blink++ > 4)
	{
		blink = 0;
		TIMSK = (0 << TOIE0);		//Stop Timer
			flag |= FLAG_SEC;
		    if (nCounter++ > TIMEOUT)
			      nCounter = 0;
	}		
}
//-------------------------------------------------------------------------//
void init_timmer0 (void)
{
	TCCR0 = (1 << CS02) | (0 << CS01) | (1 << CS00);
	TIMSK = _BV(TOIE0);
}
//------------------------------------------------------------------------//
void pwm_init (void)
{
// use OC1A pin 
    DDRB = _BV(PB1);
//
//	clear OC1A/OC1B on compare match 
// 	set OC1A/OC1B at BOTTOM, non-inverting mode
//	Fast PWM, 8bit
    TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(WGM10);
    
// 	Fast PWM, 8bit
//	Prescaler: clk/1 = 8MHz
//	PWM frequency = 8MHz / (255 + 1) = 31.25kHz
    TCCR1B = _BV(WGM12) | _BV(CS10);
    
// set initial duty cycle to zero 
    OCR1A = 0;	
}
//------------------------------------------------------------------------//
void init_adc (void)
{
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // Set ADC prescalar to 128 - 125KHz sample rate @ 16MHz
	ADMUX   = (1 << REFS0); // Set ADC reference to AVCC
	ADMUX  |= (1 << ADLAR); // Left adjust ADC result to allow easy 8 bit reading
}
//------------------------------------------------------------------------//
// Read the AD conversion result
uint16_t read_adc(uint8_t ch)
{
    ADMUX   = _BV(REFS0) | ch;
    ADCSRA |= _BV(ADEN) | _BV(ADSC) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);

		while (ADCSRA & _BV(ADSC));

    return ADCW;
}
//------------------------------------------------------------------------//
//jnut say please dot' change
#define 	VOLT_REF		17.85
#define 	CURRENT_REF		2.40
#define 	BIT				1023
double calculate_volt (uint16_t vol)
{
	return (VOLT_REF * vol)/BIT;
}
//------------------------------------------------------------------------//
double  calculate_current (uint16_t cur)
{
	return (CURRENT_REF * cur)/BIT;
}
//------------------------------------------------------------------------//
void delay_adc (uint8_t u)
{
	for (;u > 0;u--)
		_delay_ms (500);
}
//------------------------------------------------------------------------//
void display (double vol, double current)
{
	uint8_t lcdbuf[17],
			ntmp[10],len;
			
			memset  (lcdbuf,0, sizeof lcdbuf);
			sprintf (lcdbuf,"V=              ");
			sprintf (ntmp,"%02.1lfV",vol);	len = strlen (ntmp);
			memcpy (lcdbuf + (2 + (5-len)), ntmp, len);
			
			memset (ntmp, 0, sizeof ntmp);
			if (current < 1.0)
			{
				sprintf (ntmp, "%03.1lfmA",current*1000); len = strlen (ntmp);
			}
			else
			{
				sprintf (ntmp, "%02.2lfA",current); len = strlen (ntmp);
			}
			memcpy (lcdbuf + (9 + (7-len)), ntmp, len);
			lcd_gotoxy (2,1);
			lcd_printf (lcdbuf);
}
//------------------------------------------------------------------------//
void display_nobat (void)
{
	lcd_gotoxy (1,6);
	lcd_printf ("No Battery \nCheck your Batt ");
	
	flag &= ~FLAG_BATTERY;
}
//------------------------------------------------------------------------//
void display_precharge (void)
{
	lcd_gotoxy (1,6);
	lcd_printf ("Soft Charge");
}
//------------------------------------------------------------------------//
//#define 	I_INIT_V1		0.390   //390 may be Error 10%
#define		I_INIT_V2		2.2
#define 	I_FULL			100		//about 10% of Imax

#define 	V_FULL			15.0
#define 	V_STANDBY		13.8
#define 	V_CHARGE		14.7   //between 14.4-15.0
#define 	V_BATT			12.8
//------------------------------------------------------------------------//
void display_percent (double i)
{
	uint8_t lcdbuf[17],ntmp[3],
			len;
	double percent = 0;
	
		i = (i*100); 
		
			percent = (double)(100-((i*100)/((I_INIT_V2)*100.0)));
			percent += 15;  //15% of imax = full
			
			if (percent > 99)
				percent = 99;
		
		sprintf (lcdbuf, "Charge    %c",'%');
		sprintf (ntmp,"%d",(uint8_t)(percent)); len = strlen(ntmp);
		memcpy (lcdbuf + 8 + (2 - len),ntmp,len);
		lcd_gotoxy (1,6);
		lcd_printf (lcdbuf);
}
//------------------------------------------------------------------------//
int main(void)
{	
	double _volt,
			_current;
				
	batt = 0;
	
	pwm_init ();
	
	BUZ_DDR |= _BV(BUZZER);
	
	ser_init();						//init serial port
	
	sei();							//enable interrupt
	
	init_lcd ();
	
	init_timmer0 ();

	DDRC |= _BV(2);
	
	icon ();
	
	Buzzer_on (); _delay_ms (400);
	Buzzer_off ();
	
	flag |= FLAG_BATTERY;
	
	display_icon (0);
	OCR1A = 0;	flag |= FLAG_SEC;
	
	blink = 10; nCounter = 10;	//start from jnut 
	
	OCR1A = 0x10;
	
	
	
	while (1)
	{
		if (flag & FLAG_SEC)
		{
			flag &= ~FLAG_SEC;							//Remove
			_volt = calculate_volt (read_adc (VOLT));
			delay_adc (1);			
			
			_current = calculate_current (read_adc (CURRENT));
			delay_adc (1);
			
			if (nCounter > (TIMEOUT -1 ))
			{
				if (_current < 10.0)
				{
					if ((_volt > 15) && (_current < 10))
					{	display_nobat ();
						display_icon (0);
						flag &= ~FLAG_BATTERY;
						OCR1A = 0x10;
					}
					else
					{
						flag |= FLAG_BATTERY;
						display_icon (batt++);
						display (_volt,_current);
						
						/****** charge here ******/
						
						if (_volt < V_CHARGE)
						{
							if (_current < I_INIT_V2)
								  OCR1A +=1;
							if ((_volt >= V_STANDBY) && (_current < 390))
								display_percent (_current);
							else
								display_precharge ();
						}
						else
						{
							if (_volt > V_BATT)
							{
								display_percent (_current);
								//check Full Charger here
								if ((_volt > V_FULL) && (_current < I_FULL))
								{
									//Full Charge
									//check VFull and adjust OCR1A = 13.8 Volt
								}
							}
						}
					}
				}
			}
		TIMSK = _BV(TOIE0);		//Enable Timer Again
		}
	}
	while (1);
}
//------------------------------------------------------------------------//
//------------------------------------------------------------------------//
//------------------------------------------------------------------------//
//------------------------------------------------------------------------//
